package db

import (
	jsoniter "github.com/json-iterator/go"
	"github.com/prologic/bitcask"
	"github.com/rs/zerolog/log"
	"jupemail/config"
	"time"
)

var (
	Storage    *bitcask.Bitcask
	maxSizeObj uint64
	dataDir    string
	err        error
	key        []byte
)

type Key struct {
	Owner string
	Time  int64
}

const megabyte = float64(1024 * 1024)

func DbInit() {
	dataDir = config.Config.GetString("data.directory")
	maxSizeObj = config.Config.GetUint64("email.maxsize")
	var dbo = []bitcask.Option{
		bitcask.WithMaxValueSize(maxSizeObj * uint64(1024)),
	}
	if Storage, err = bitcask.Open(dataDir+"jupemail_data", dbo...); err != nil {
		log.Fatal().Err(err).Msg("FUBAR'd")
	}

}

func CompressAndStore(rcpt string, valdata []byte) error {
	var json = jsoniter.ConfigCompatibleWithStandardLibrary
	key, err = json.Marshal(&Key{
		Owner: rcpt,
		Time:  time.Now().Unix(),
	})
	if err != nil {
		return err
	}
	data, err := Gzip(valdata)
	if err != nil {
		return err
	}
	return Storage.Put(key, data)
}
